
//
// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
// 
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.
//

// This source file was created by a code generator.
// Please do not edit.

#include "qpid/management/Manageable.h"
#include "qpid/management/Buffer.h"
#include "qpid/types/Variant.h"
#include "qpid/amqp_0_10/Codecs.h"
#include "qpid/agent/ManagementAgent.h"
#include "Host.h"

#include <iostream>
#include <sstream>

using namespace qmf::com::redhat::matahari;
using           qpid::management::ManagementAgent;
using           qpid::management::Manageable;
using           qpid::management::ManagementObject;
using           qpid::management::Args;
using           qpid::management::Mutex;
using           std::string;

string  Host::packageName  = string ("com.redhat.matahari");
string  Host::className    = string ("host");
uint8_t Host::md5Sum[MD5_LEN]   =
    {0x1a,0xeb,0x2,0x4,0xc3,0xf0,0x4,0x40,0x82,0x79,0xf3,0x5c,0xdf,0xc8,0xab,0x55};

Host::Host (ManagementAgent*, Manageable* _core) :
    ManagementObject(_core)
{
    
    uuid = "";
    hostname = "";
    memory = 0;
    hypervisor = "";
    arch = "";
    beeping = 0;



}

Host::~Host ()
{

}

namespace {
    const string NAME("name");
    const string TYPE("type");
    const string ACCESS("access");
    const string IS_INDEX("index");
    const string IS_OPTIONAL("optional");
    const string UNIT("unit");
    const string MIN("min");
    const string MAX("max");
    const string MAXLEN("maxlen");
    const string DESC("desc");
    const string ARGCOUNT("argCount");
    const string ARGS("args");
    const string DIR("dir");
    const string DEFAULT("default");
}

void Host::registerSelf(ManagementAgent* agent)
{
    agent->registerClass(packageName, className, md5Sum, writeSchema);
}

void Host::writeSchema (std::string& schema)
{
    const int _bufSize=65536;
    char _msgChars[_bufSize];
    ::qpid::management::Buffer buf(_msgChars, _bufSize);
    ::qpid::types::Variant::Map ft;

    // Schema class header:
    buf.putOctet       (CLASS_KIND_TABLE);
    buf.putShortString (packageName); // Package Name
    buf.putShortString (className);   // Class Name
    buf.putBin128      (md5Sum);      // Schema Hash
    buf.putShort       (6); // Config Element Count
    buf.putShort       (0); // Inst Element Count
    buf.putShort       (2); // Method Count

    // Properties
    ft.clear();
    ft[NAME] = "uuid";
    ft[TYPE] = TYPE_SSTR;
    ft[ACCESS] = ACCESS_RO;
    ft[IS_INDEX] = 1;
    ft[IS_OPTIONAL] = 0;
    ft[DESC] = "Host UUID";
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "hostname";
    ft[TYPE] = TYPE_SSTR;
    ft[ACCESS] = ACCESS_RO;
    ft[IS_INDEX] = 0;
    ft[IS_OPTIONAL] = 0;
    ft[DESC] = "Hostname";
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "memory";
    ft[TYPE] = TYPE_S32;
    ft[ACCESS] = ACCESS_RO;
    ft[IS_INDEX] = 0;
    ft[IS_OPTIONAL] = 0;
    ft[UNIT] = "kb";
    ft[DESC] = "Amount of primary memory for host (kb)";
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "hypervisor";
    ft[TYPE] = TYPE_SSTR;
    ft[ACCESS] = ACCESS_RO;
    ft[IS_INDEX] = 0;
    ft[IS_OPTIONAL] = 0;
    ft[DESC] = "Hypervisor used by host";
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "arch";
    ft[TYPE] = TYPE_SSTR;
    ft[ACCESS] = ACCESS_RO;
    ft[IS_INDEX] = 0;
    ft[IS_OPTIONAL] = 0;
    ft[DESC] = "Architecture of host";
    buf.putMap(ft);

    ft.clear();
    ft[NAME] = "beeping";
    ft[TYPE] = TYPE_BOOL;
    ft[ACCESS] = ACCESS_RW;
    ft[IS_INDEX] = 0;
    ft[IS_OPTIONAL] = 0;
    ft[DESC] = "speaker beep loop active? (used to identify host)";
    buf.putMap(ft);


    // Statistics

    // Methods
    ft.clear();
    ft[NAME] =  "shutdown";
    ft[ARGCOUNT] = 0;
    ft[DESC] = "Shutdown node";
    buf.putMap(ft);

    ft.clear();
    ft[NAME] =  "reboot";
    ft[ARGCOUNT] = 0;
    ft[DESC] = "Reboot node";
    buf.putMap(ft);


    {
        uint32_t _len = buf.getPosition();
        buf.reset();
        buf.getRawData(schema, _len);
    }
}




std::string Host::getKey() const
{
    std::stringstream key;

    key << uuid;
    return key.str();
}



void Host::mapEncodeValues (::qpid::types::Variant::Map& _map,
                                              bool includeProperties,
                                              bool includeStatistics)
{
    using namespace ::qpid::types;
    Mutex::ScopedLock mutex(accessLock);

    if (includeProperties) {
        configChanged = false;
    _map["uuid"] = ::qpid::types::Variant(uuid);
    _map["hostname"] = ::qpid::types::Variant(hostname);
    _map["memory"] = ::qpid::types::Variant(memory);
    _map["hypervisor"] = ::qpid::types::Variant(hypervisor);
    _map["arch"] = ::qpid::types::Variant(arch);
    _map["beeping"] = ::qpid::types::Variant(beeping);

    }

    if (includeStatistics) {
        instChanged = false;






    // Maintenance of hi-lo statistics


    }
}

void Host::mapDecodeValues (const ::qpid::types::Variant::Map& _map)
{
    ::qpid::types::Variant::Map::const_iterator _i;
    Mutex::ScopedLock mutex(accessLock);

    if ((_i = _map.find("uuid")) != _map.end()) {
        uuid = (_i->second).getString();
    }
    if ((_i = _map.find("hostname")) != _map.end()) {
        hostname = (_i->second).getString();
    }
    if ((_i = _map.find("memory")) != _map.end()) {
        memory = _i->second;
    }
    if ((_i = _map.find("hypervisor")) != _map.end()) {
        hypervisor = (_i->second).getString();
    }
    if ((_i = _map.find("arch")) != _map.end()) {
        arch = (_i->second).getString();
    }
    if ((_i = _map.find("beeping")) != _map.end()) {
        beeping = _i->second;
    }

}

void Host::doMethod (string& methodName, const ::qpid::types::Variant::Map&, ::qpid::types::Variant::Map& outMap)
{
    Manageable::status_t status = Manageable::STATUS_UNKNOWN_METHOD;
    std::string          text;


    if (methodName == "shutdown") {
        ::qpid::management::ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_SHUTDOWN, ioArgs, text);
        outMap["_status_code"] = (uint32_t) status;
        outMap["_status_text"] = ::qpid::management::Manageable::StatusText(status, text);
        return;
    }

    if (methodName == "reboot") {
        ::qpid::management::ArgsNone ioArgs;
        status = coreObject->ManagementMethod (METHOD_REBOOT, ioArgs, text);
        outMap["_status_code"] = (uint32_t) status;
        outMap["_status_text"] = ::qpid::management::Manageable::StatusText(status, text);
        return;
    }

    outMap["_status_code"] = (uint32_t) status;
    outMap["_status_text"] = Manageable::StatusText(status, text);
}
